<?php

namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use \Teknisa\Libs\Util\Utilities as LibUtilities;

class IntegracaoMercadumController
{

  	protected $connection;
  	protected $environment;
  	protected $utilities;
  	
    public function __construct($connection, $environment, LibUtilities $utilities){
        $this->connection  = $connection;
        $this->environment = $environment;
        $this->utilities   = $utilities;
    }
    
    public function enviaDadosIntegracao ($dados, $url){
        try{
            $postdata = http_build_query($dados);
            $ch = curl_init();
            curl_setopt($ch,CURLOPT_URL, $url);
            curl_setopt($ch,CURLOPT_POST, true);
            curl_setopt($ch,CURLOPT_POSTFIELDS, $postdata);
            curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
            $result = curl_exec($ch);
        } catch (Exception $e) {
            $error = new DTO\Response\Error($e->getMessage(), $e->getCode());
              $response->setError($error);
        }
    }

    private function getTeknisaOrganizationId() {
        $userData       = $this->getUserData();
        $oAccCustom     = $this->readOAccCustom();
        $nrOrgOper      = $userData['NRORGOPER'];
        $organizationId = $userData['ORGANIZATION_ID'];
        
        if(!empty($nrOrgOper)) {
            return $nrOrgOper;
        }
        if(!empty($oAccCustom)) {
            return $oAccCustom['organizationId'];
        }
        return $organizationId;
    }
    
    private function getUserData(){
        $cdoperador = $this->environment->getCdOperador();
        $sql = "SELECT NRORGOPER, NRORG AS ORGANIZATION_ID FROM OPERADOR WHERE CDOPERADOR = :P_CDOPERADOR";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            'P_CDOPERADOR' => $cdoperador
        ));
        return $stmt->fetch();
    }

    protected function readOAccCustom() {
        $oAccCustom = array();
        $occCustomPath = InstanceManager::getInstance()->getParameter('oAccCustom');
        $modulesPath = __DIR__ . '/../../../../../..';
        $loginPath = $modulesPath . '/login/backend/config/';   
        $fileDir = $loginPath . $occCustomPath;
        try {
            if (file_exists($fileDir)) {
                $fileContents  = file_get_contents($fileDir);
                $fileDecrypted = LibUtilities::zeedhiDecrypt($fileContents, LibUtilities::LOGIN_CRYPT_STRING);
                $fileDecoded   = @json_decode($fileDecrypted, true);
                if(isset($fileDecoded['organizationId']) && 
                    isset($fileDecoded['dbHost']) && 
                    is_numeric($fileDecoded['organizationId'])) {
                    $oAccCustom = $fileDecoded;
                }
            }
        } catch(\Exception $e) {}

        return $oAccCustom;
    }
    
    public function getDstokenintgeco(){
        $sql = "SELECT PARAM.DSTOKENINTGECO FROM PARAMMERCADUM PARAM, ORGANIZACAO ORG WHERE ORG.NRORG = PARAM.NRORG";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute();
        return $stmt->fetch();
    }


    // Supplier
    public function integracaoMercadumSupplier ($row){
        try{
            $fornecedores = $this->selectFornecedores();
            $mercadumIntegration = $this->buildArrayFornecedores($row,$fornecedores);
            $url = 'http://paulooliveira1.zeedhi.com/workfolder/mercadum/mercadum-fornecedor/backend/api/integration/suppliers';
            $return = $this->enviaDadosIntegracao($mercadumIntegration, $url);
        } catch (Exception $e) {
            $error = new DTO\Response\Error($e->getMessage(), $e->getCode());
		    $response->setError($error);
        }
    }
    
    public function selectFornecedores (){
        try {
            $query = "  SELECT  DF.NMUSUARIOECO, DF.CDOPERFORN, DF.DSLOGINECO, F.CDFORNECED, F.IDTPIJURFORN,
                                F.NRINSJURFORN, F.NMRAZSOCFORN, F.NMFANTFORN, F.NRINSESTFORN, E.IDTPENDEFORN, F.VRMINAUTFOR, F.VRPERVARDESCPREC, F.VRPERVARPREC,
                                E.CDPAIS, E.SGESTADO, E.CDMUNICIPIO, E.DSENDEFORN, E.NMBAIRFORN, E.NRCEPFORN
                        FROM FORNECEDOR F 
                                INNER JOIN ENDEFORN E ON F.CDFORNECED = E.CDFORNECED
                                INNER JOIN DADOSFORNECOMMERCE DF ON F.CDFORNECED = DF.CDFORNECED
                        WHERE E.IDTPENDEFORN = :P_IDTPENDEFORN";
                    
            $params = array(
                "P_IDTPENDEFORN" => "P"
            );        
            
            $std = $this->connection->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function buildArrayFornecedores($row, $fornecedores){
		$mercadumIntegration = array();
		$dados = array();		
		$dados['Nrorg'] = $row['NRORG'] == "1" ? $this->getTeknisaOrganizationId() : $row['NRORG'];
		$dados['Chave'] = $row['CHAVE'];
		
		if (count($fornecedores) > 0) {
			$fornecedoresList = array();
			foreach ($fornecedores as $fornecedor) {
			    $dadosFornecedor = array();
				$dadosFornecedor['SUPPLIER']         =  $this->buildSupplier($fornecedor);
				$dadosFornecedor['SUPPLIER_ADDRESS'] =  $this->buildSupplierAddress($fornecedor);
				$dadosFornecedor['USERS']            =  $this->buildUsers($fornecedor);
				array_push($fornecedoresList, $dadosFornecedor);
			}
    	    $dados['Fornecedores'] = $fornecedoresList;
    	    $mercadumIntegration["mercadumIntegration"] = $dados;
			return $mercadumIntegration;
		} else {
			throw new \Exception("Não foram encontrados Fornecedores.");
		}
    }
    
    public function buildSupplier($fornecedor){
		$supplier = array(
			"SUPPLIER_ID"        => $fornecedor['CDFORNECED'],
			"TYPE_SUPPLIER"      => $fornecedor['IDTPIJURFORN'],
			"REGISTRATION"       => $fornecedor['NRINSJURFORN'],
			"CORPORATE_NAME"     => $fornecedor['NMRAZSOCFORN'],
			"SUPPLIER_NAME"      => $fornecedor['NMFANTFORN'],
			"STATE_REGISTRATION" => $fornecedor['NRINSESTFORN'],
			"MINIMUM_BILLING"    => $fornecedor['VRMINAUTFOR'],
			"DISCOUNT_VALUE"     => $fornecedor['VRPERVARDESCPREC'],
			"ADDITION_VALUE"     => $fornecedor['VRPERVARPREC']
		);
		return $supplier;
    }
     
    public function buildSupplierAddress($fornecedor){
		$supplierAddress = array(
			"SUPPLIER_ADDRESS_ID" => $fornecedor['IDTPENDEFORN'],
			"COUNTRY_CODE"        => $fornecedor['CDPAIS'],
			"STATE_CODE"          => $fornecedor['SGESTADO'],
			"CITY_CODE"           => $fornecedor['CDMUNICIPIO'],
			"ADDRESS"             => $fornecedor['DSENDEFORN'],
			"DISTRICT"            => $fornecedor['NMBAIRFORN'],
			"ZIP_CODE"            => $fornecedor['NRCEPFORN']
		); 
		return $supplierAddress;
    }
    
    public function buildUsers($fornecedor){
		$users = array(
			"USER_NAME"  => $fornecedor['NMUSUARIOECO'],
			"USER_ID"    => $fornecedor['CDOPERFORN'],
			"EMAIL"      => $fornecedor['DSLOGINECO']
		);
		return $users;
    }
    //

    
    // Pedido de Coleta
    public function integracaoMercadumPedidoColeta ($row){
        try{
            $pedidos         = $this->selectPedidoColeta($row);
            $nrpedidocol     = $this->buildNRPEDIDOCOL($pedidos);
            $pedidoList = array();
            foreach ($nrpedidocol as $nrpedido){
                $row['NRPEDIDOCOL'] = $nrpedido;
                $ItensPedidoColeta = $this->selectItensPedidoColeta($row); 
                $pedidoList = $this->buildArray_PedidoList($pedidoList,$ItensPedidoColeta);
                $mercadumIntegration = $this->buildArray_PedidoColeta($row, $pedidoList);
                //$url = '';
                //$return = $this->enviaDadosIntegracao($mercadumIntegration, $url);
                $return = true;
                if($return){
                    $this->updateSituacaoPedidoColeta($row, $nrpedido);
                }
                $pedidoList = [];
                $mercadumIntegration = [];
            }
        } catch (Exception $e) {
            $error = new DTO\Response\Error($e->getMessage(), $e->getCode());
		    $response->setError($error);
        }
    }
    
    public function buildArray_PedidoColeta($row, $pedidoList){
		$arrayMercadum = array();
		$dados         = array();
		
		$dados['Nrorg'] = $row['NRORG'] == "1"  ? $this->getTeknisaOrganizationId()            : $row['NRORG'];
		$dados['Chave'] = !isset($row['CHAVE']) ? $this->getDstokenintgeco()['DSTOKENINTGECO'] : $row['CHAVE'];
        $dados['PedidosColeta'] = $pedidoList;
        $arrayMercadum["mercadumIntegration"] = $dados;
        
		return $arrayMercadum;
    }    
    
    public function buildArray_PedidoList($pedidoList, $itens){
		$pedido        = array();
		$dadosPedido   = array();
		$itensPedido   = array();
		
		foreach ($itens as $item) {
            $novoDadosPedido = $this->buildArray_RequestQuote($item);
            $itemPedido      = $this->buildArray_QuotationItems($item);
            $ultimoItem      = array_slice($itens, -1, 1)[0];
            
            if($dadosPedido == []){
			    $dadosPedido = $novoDadosPedido;
            } else if ($dadosPedido['REQUEST_NUMBER'] != $novoDadosPedido['REQUEST_NUMBER']){
                $pedido['REQUEST_QUOTE']   = $dadosPedido;
                $pedido['QUOTATION_ITEMS'] = $itensPedido;
                array_push($pedidoList, $pedido);
                $pedido      = [];  
                $itensPedido = [];
			    $dadosPedido = $novoDadosPedido;
            }
            array_push($itensPedido, $itemPedido);
			if($item == $ultimoItem){
        	    $pedido['REQUEST_QUOTE']   = $dadosPedido;
        	    $pedido['QUOTATION_ITEMS'] = $itensPedido;
			    array_push($pedidoList, $pedido);
		    }
		}
		
		return $pedidoList;
    }
    
    public function buildNRPEDIDOCOL($pedidos, $maximo = 20){
		$NRPEDIDOCOL = array();
		$contador = 1;
		$indice = 0;
		$string = "";
		
		if (count($pedidos) > 0) {
			foreach ($pedidos as $pedido) {
                $ultimoPedido = array_slice($pedidos, -1, 1)[0];
			    $string = $string.'_'.$pedido['NRPEDIDOCOL'];
			    $contador = $contador + 1;
			    if($contador > $maximo || $pedido == $ultimoPedido){
			        $NRPEDIDOCOL[$indice] = $string;
			        $string = "";
			        $indice = $indice + 1;
			        $contador = 1;
			    }
			}
		} else {
			throw new \Exception("Não foram encontrados Pedidos de Coleta.");
		}
		return $NRPEDIDOCOL;
    }
    
    public function buildArray_RequestQuote($pedido){
		$request_quote = array(
			"BRANCH_ID"          => $pedido['CDFILIAL'],
			"ORGANIZATION_ID"    => $pedido['NRORG'],
			"SUPPLIER_ID"        => $pedido['CDFORNECED'],
			"RESPONSIBLE_ID"     => $pedido['COD_RESPONSAVEL'],
			"DATE_RELEASE"       => $pedido['DTPEDCOLGERAL'],
			"VALIDITYINI_DATE"   => $pedido['DTINIPERGERAL'],
			"VALIDITYFIN_DATE"   => $pedido['DTFINPERGERAL'],
			"EXPIRATION_DATE"    => $pedido['DTEXPIRAPEDCOL'],
			"STATUS"             => $pedido['IDSITUPEDCOLETA'],
			"REQUEST_NUMBER"     => $pedido['NRPEDIDOCOL']
		);
		return $request_quote;
    }
    
    public function buildArray_QuotationItems($pedido){
		$quotationItems = array(
			"PRODUCT_INCLUDED"   => $pedido['IDPRODINCLFOR'],
			"REQUEST_NUMBER"     => $pedido['NRPEDIDOCOL'],
			"BRANCH_ID"          => $pedido['CDFILIAL'],
			"ORGANIZATION_ID"    => $pedido['NRORG'],
			"PRODUCT_ID"         => $pedido['CDPRODUTO'],
			"BRAND_CODE"         => $pedido['CDMARCA'],
			"QUANTITY_REQUESTED" => $pedido['QTSOMA'],
			"PRICE"              => $pedido['VRPRECUNIPRMARC'],
			"PREVIOUS_PRICE"     => $pedido['VRULTPRECPRMARC'],
			"OBJECTIVE_PRICE"    => $pedido['PRECOOBJETIVO'],
			"TRADING_PRICE"      => $pedido['VARIACAOPRECO'],
			"READJUSTED_ID"      => $pedido['IDITEMPEDCOLALT'],
			"NEGOTIATION_ID"     => $pedido['IDRENEGOCIAPCOL'],
			"NOTE_NEGOTIATION"   => $pedido['DSOBSRENEGOCIA']
		);
		return $quotationItems;
    }
    
    public function selectPedidoColeta ($row){
        try {
            $query = "  SELECT PG.CDFILIAL, PG.NRPEDIDOCOL
                        FROM PEDCOLGERAL PG
                        WHERE PG.IDINTEGRAMCD = :P_IDINTEGRAMCD
                           AND PG.CDFILIAL = :P_CDFILIAL
                           AND PG.NRORG = :P_NRORG
                        ORDER BY NRPEDIDOCOL ASC
                     ";
                    
            $params = array(
                "P_NRORG"        => $row["NRORG"],
                "P_CDFILIAL"     => $row["CDFILIAL"],
                "P_IDINTEGRAMCD" => "N"
            );        
            
            $std = $this->connection->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function selectItensPedidoColeta ($row){
        try {
            $query = "  SELECT DISTINCT 
                          PRMARCPEDCOL.IDPRODINCLFOR, /*PRODUCT_INCLUDED*/
                          PRMARCPEDCOL.CDFILIAL, /*BRANCH_ID*/
                          PRMARCPEDCOL.NRPEDIDOCOL, /*REQUEST_NUMBER*/
                          SUBSELECT.CDFORNECED, /*SUPPLIER_ID*/
                          PRMARCPEDCOL.CDMARCA, /*BRAND_CODE*/
                          ROUND(PRMARCPEDCOL.VRPRECUNIPRMARC,3) VRPRECUNIPRMARC, /*PRICE*/
                          ROUND(PRMARCPEDCOL.VRULTPRECPRMARC,3) VRULTPRECPRMARC, /*PREVIOUS_PRICE*/
                          PRODUTO.CDPRODUTO, /*PRODUCT_ID*/
                          PRMARCPEDCOL.VRPRECOOBJETIVO PRECOOBJETIVO, /*OBJECTIVE_PRICE*/
                          ROUND( (PRMARCPEDCOL.VRPRECUNIPRMARC / PRMARCPEDCOL.VRULTPRECPRMARC * 100) - 100,2) AS PERCVARIACAO,
                          SUBSELECT.QTSOMA, /*QUANTITY_REQUESTED*/
                          SUBSELECT.IDITEMPEDCOLALT, /*READJUSTED_ID*/
                          PRMARCPEDCOL.DSOBSRENEGOCIA, /*NOTE_NEGOTIATION*/
                          PEDCOLGERAL.IDRENEGOCIAPCOL, /*NEGOTIATION_ID*/
                          PEDCOLGERAL.DTPEDCOLGERAL, /*DATE_RELEASE*/
                          PEDCOLGERAL.IDSITUPEDCOLETA, /*STATUS*/
                          PRMARCPEDCOL.NRORG, /*ORGANIZATION_ID*/
                          PEDCOLGERAL.DTINIPERGERAL, /*VALIDITYINI_DATE*/
                          PEDCOLGERAL.DTFINPERGERAL, /*VALIDITYFIN_DATE*/
                          PEDCOLGERAL.CDOPERLANC COD_RESPONSAVEL, /*RESPONSIBLE_ID*/
                          PEDCOLGERAL.DTEXPIRAPEDCOL, /*EXPIRATION_DATE*/
                          
                          DECODE(ROUND( ( ( (NVL(PRMARCPEDCOL.VRPRECREN,PRMARCPEDCOL.VRPRECUNIPRMARC) / PRMARCPEDCOL.VRULTPRECPRMARC) - 1) * 100),2), -100, 0, ROUND( ( ( (NVL(PRMARCPEDCOL.VRPRECREN,PRMARCPEDCOL.VRPRECUNIPRMARC) / PRMARCPEDCOL.VRULTPRECPRMARC) - 1) * 100),2)) 
                            AS VARIACAOPRECO /*TRADING_PRICE*/
                        
                        FROM PEDCOLGERAL 
                            INNER JOIN PRMARCPEDCOL  ON PEDCOLGERAL.NRPEDIDOCOL = PRMARCPEDCOL.NRPEDIDOCOL
                                                    AND PEDCOLGERAL.CDFILIAL    = PRMARCPEDCOL.CDFILIAL
                                                    AND PEDCOLGERAL.NRORG       = PRMARCPEDCOL.NRORG
                             INNER JOIN PRODUTO ON PRMARCPEDCOL.CDPRODUTO = PRODUTO.CDPRODUTO
                                               AND PRMARCPEDCOL.NRORG     = PRODUTO.NRORG
                              LEFT JOIN (   SELECT SUM(NVL(QTPRODPEDCOL,0)) QTSOMA, 
                                                 NVL(ITEMPEDCOL.VRPERDESPEDCOL,0) AS VRPERDESPEDCOL,
                                                 ITEMPEDCOL.CDPRODUTO,
                                                 ITEMPEDCOL.CDMARCA,
                                                 ITEMPEDCOL.IDITEMPEDCOLALT,
                                                 ITEMPEDCOL.CDFORNECED
                                            FROM ITEMPEDCOL
                                            WHERE ITEMPEDCOL.CDFILIAL = :P_CDFILIAL
                                                AND ITEMPEDCOL.NRORG    = :P_NRORG
                                                AND EXISTS ( SELECT 1
                                                             FROM PEDCOLETA
                                                             WHERE PEDCOLETA.CDFILIAL    = ITEMPEDCOL.CDFILIAL
                                                                AND PEDCOLETA.NRPEDCOLETA = ITEMPEDCOL.NRPEDCOLETA
                                                                AND PEDCOLETA.CDFORNECED  = ITEMPEDCOL.CDFORNECED
                                                                AND PEDCOLETA.CDREGIAO    = ITEMPEDCOL.CDREGIAO
                                                                AND PEDCOLETA.NRORG       = ITEMPEDCOL.NRORG
                                                                AND INSTR(:P_NRPEDIDOCOL, PEDCOLETA.NRPEDIDOCOL) > 0
                                                                AND PEDCOLETA.CDFILIAL    = :P_CDFILIAL
                                                                AND PEDCOLETA.NRORG       = :P_NRORG)         
                                               GROUP BY NVL(ITEMPEDCOL.VRPERDESPEDCOL,0),
                                                        ITEMPEDCOL.CDPRODUTO,
                                                        ITEMPEDCOL.CDMARCA,
                                                        ITEMPEDCOL.IDITEMPEDCOLALT,
                                                        ITEMPEDCOL.CDFORNECED) SUBSELECT ON PRMARCPEDCOL.CDPRODUTO = SUBSELECT.CDPRODUTO
                                                                                        AND PRMARCPEDCOL.CDMARCA   = SUBSELECT.CDMARCA
                        WHERE UPPER(PRODUTO.NMPRODUTO) NOT LIKE '%(INATIVO)%'
                           AND INSTR(:P_NRPEDIDOCOL, PEDCOLGERAL.NRPEDIDOCOL) > 0
                           AND PEDCOLGERAL.CDFILIAL     = :P_CDFILIAL
                           AND PEDCOLGERAL.NRORG        = :P_NRORG
                           AND PEDCOLGERAL.IDINTEGRAMCD = 'N'  
                        ORDER BY NRPEDIDOCOL   ";
                    
            $params = array(
                "P_NRORG"       => $row["NRORG"],
                "P_CDFILIAL"    => $row["CDFILIAL"],
                "P_NRPEDIDOCOL" => $row["NRPEDIDOCOL"]
            );        
            
            $std = $this->connection->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function updateSituacaoPedidoColeta($row, $nrpedido, $situacao = 'S'){
        try {
            $query = "  
                        UPDATE PEDCOLGERAL 
                        SET IDINTEGRAMCD = :P_IDINTEGRAMCD 
                        WHERE CDFILIAL = :P_CDFILIAL 
                            AND NRORG = :P_NRORG 
                            AND INSTR(:P_NRPEDIDOCOL, PEDCOLGERAL.NRPEDIDOCOL) > 0
                     ";
                    
            $params = array(
                "P_NRORG"        => $row["NRORG"],
                "P_CDFILIAL"     => $row["CDFILIAL"],
                "P_NRPEDIDOCOL"  => $nrpedido,
                "P_IDINTEGRAMCD" => $situacao
            );        
            
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e) {
            throw $e;
        }
    }
    //
}
