<?php
namespace Modules\Cotacao\Modules\ListaPreco\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class Sup01300VerificacaoListaPrecoController extends \Zeedhi\Framework\Controller\Report {
    
    protected $entityManager;
    protected $birtStrategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    protected $Sup01300VerificacaoListaPrecoService;
    
    public function __construct($reportService, $entityManager, $birtStrategy, $newCode, $environment, $connection, $dataSourceManager, $Sup01300VerificacaoListaPrecoService) {
        
        $this->entityManager     = $entityManager;
        $this->birtStrategy      = $birtStrategy;
        $this->reportService     = $reportService;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->Sup01300VerificacaoListaPrecoService = $Sup01300VerificacaoListaPrecoService;
    }
    
    public function filial() {
        $params = array(
            "CDFILIAL" => $this->environment->getCdFilial()
        );
        $sql = "SELECT NVL(CDFILCENTCMP, CDFILIAL) AS CDFILCENTCMP FROM FILIAL WHERE (CDFILIAL = :CDFILIAL)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll()[0]['CDFILCENTCMP'];
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $row['CDFILIAL'] = $this->Sup01300VerificacaoListaPrecoService->getCdFilial($this->environment->getCdFilial());
        
        if($row['CDOPERADOR'] != 'T'){
             $nrlistaForn = $this->criarListaValores($row['CDOPERADOR']);
             $row['CDOPERADOR'] = "_NRLISTA=".$nrlistaForn;
        }
        
        if($row['CDREGIAO'] != 'T'){
            $nrlistaForn = $this->criarListaValores($row['CDREGIAO']);
            $row['CDREGIAO'] = "_NRLISTA=".$nrlistaForn;
        }
        
        $row['CDOPERADOR1'] = $row['CDOPERADOR'];
        $row['CDOPERADOR2'] = $row['CDOPERADOR'];
        $row['CDOPERADOR3'] = $row['CDOPERADOR'];
        $row['CDREGIAO1'] = $row['CDREGIAO'];
        
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $this->stNomeRel = $this->getReportName($row);
         $this->reportMapping[$this->stNomeRel] =
            array (
                'strategy' => $row['STRATEGY'],
                'parameterMapping' =>
                    array (
                        "CDREGIAO"       => "L_CDREGIAO1",
                        "CDREGIAO1"      => "L_CDREGIAO2",
                        "CDARVPRODINI"   => "P_CDINI",
                        "CDARVPRODFIN"   => "P_CDFIN",
                        "CDFILIAL"       => "P_CDFILIAL",
                        //"CDFILCENTCMP" => "P_CDFILCENTCMP",
                        "CDOPERADOR"     => "L_CDOPERADOR1",
                        "CDOPERADOR1"    => "L_CDOPERADOR2",
                        "CDOPERADOR2"    => "L_CDOPERADOR3",
                        "CDOPERADOR3"    => "L_CDOPERADOR4",
                        "NRORG"          => "NRORG"
                    )
            );
    }
    
    protected function criarListaValores($lista){
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_',$lista);
        try{
            foreach($valores as $key){
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch(\Exception $e){
            throw $e;
        }
     }

     public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );

        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
     }
    
    public function updateItemcole(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $row['CDFILIAL'] = $this->Sup01300VerificacaoListaPrecoService->getCdFilial($this->environment->getCdFilial());
            $this->Sup01300VerificacaoListaPrecoService->updateItemcole($row);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
}